//********************************************************************************
//*                                                                              *
//* Description:                                                                 *
//*                                                                              *
//*        Name:  Compass.c                                                      *
//*      Author:  Daniel F. Ramirez                                              *
//*        Date:  09/08/2011                                                     *    
//*                                                                              *
//* Description:                                                                 *
//*     This application tests the Sparkfun HMC6352 I2C Electronic Compass made  *
//* by Honeywell using the VEX v0.5 Microcontroller.                             *  
//*                                                                              *                  
//********************************************************************************
  
// Include file headers here
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <i2c.h>                                        // I2C library functions
#include <pwm.h>                                        // PWM library functions
#include <timers.h>                                     // Timer library functions
#include <portb.h>                                      // PORTB library functions                                 
#include <usart.h>                                      // USART library functions
#include <delays.h>                                     // High resolution delay functions
 
//#include <delays.h>
#include "sw_i2c.h"
#include "Main.h"
   


void init_i2c(void)
{
   long mem_adr;
   byte dat, n;
   int i;
   int Direction = 0; 
   int x, y;

    //***********************************************************************************************
    // Initialize the pins used for the I2C interface

    // Disable serial port  
    RCSTA2bits.SPEN = 0;    // Serial Port disable [249]
                            //  0: serial port is disabled
                            //  1: serial port is enabled

#ifdef TEST
    //***********************************************************************************************
    // Test the Honeywell HMC6352 I2C Electronic Compass
    PrintToScreen("Test the Honeywell HMC6352 I2C Electronic Compass...\r\n");
    Wait(2000);
    PrintToScreen("trace 1\r\n");
    for (i=0; i<100; i++)
    {
        PrintToScreen("This is a test %d times\r\n",i);

    } 
 



    //****************************************************************************************************************
    //****************************************************************************************************************
    //* Compass test - Read the I2C compass
    //****************************************************************************************************************
    //****************************************************************************************************************


    //main loop code goes here 
    PrintToScreen("Read the I2C Compass and report current heading in degrees  ... \r\n");
    while(1)    
    { 
        Direction  = Get_Compass_Heading();
        PrintToScreen("Direction (deg.) = %i\n\r", Direction); 

        Wait(200);   
    }


    while(1)    
    { 
        //ClrWdt(); 
    
        PrintToScreen("Start\n\r");    
        i2c_start(); 
        //IdleI2C(); 

        // Write the data byte to the I2C bus
        //MasterWriteI2C(0x42);
        i2c_out_byte(0x42);
        //IdleI2C(); 
        i2c_nack();

        // Write the data byte to the I2C bus
        //MasterWriteI2C(0x41);
        i2c_out_byte(0x41);   //Send command "A"    
       //IdleI2C(); 
        i2c_nack();

        i2c_stop(); 
        Wait(10);    //wait at least 6000 uS 

        i2c_start();        
       //IdleI2C(); 

        // Write the data byte to the I2C bus
        //MasterWriteI2C(0x43);
        i2c_out_byte(0x43);        //Write read command 
        //IdleI2C(); 
        i2c_nack();

        Wait(10);    //wait at least 6000 uS 

        //x = MasterReadI2C();    //Read first byte 
        x = i2c_in_byte();     //Read first byte 
        //IdleI2C(); 
        //AckI2C(); 
        i2c_ack();
        //IdleI2C(); 

  
        //y = MasterReadI2C();    //Read second byte 
        y = i2c_in_byte();     //Read second byte 
        //IdleI2C(); 
        //AckI2C(); 
        //IdleI2C(); 
        //NotAckI2C(); 
        i2c_nack();
        //IdleI2C(); 
        
        i2c_stop(); 
        Direction = ((x << 8) + y) / 10;

        PrintToScreen("Direction (deg.) = %i\n\r", Direction); 

        Wait(200);        
    } 

    

//***********************************************************************************

// Write data to 24LC512 Serial I2C EEPROM


while(1)
{
    for (i=0; i<256; i++)
    //for (i=0; i<20; i++)
    {
        PrintToScreen("write: i = %d, dat = %d\r\n", i, i);
        random_write(0x00, i, 256 - (byte) i);
 
    }

    for (i=0; i<256; i++)
    //for (i=0; i<20; i++)
    {
        dat = random_read(0x00, i);
        PrintToScreen("read: i = %d, dat = %d\r\n", i, dat);

    }
    //delay_ms(500);
}

while(1)
{

   mem_adr=0x0700;
   for(n=0; n<16; n++)
   {
      dat = 0xff-n;
      PrintToScreen("trace 2\r\n");
      random_write(0x00, mem_adr, dat);
      ++mem_adr;
   }
    // now, read the data back and display
   lcd_init();
   mem_adr=0x0700;

   PrintToScreen("trace 3\r\n");

   for(n=0; n<16; n++)
   {
      if ((n!=0) && ((n%4) == 0))
      {
         PrintToScreen("/r/n");
      }
      PrintToScreen("trace 4\r\n");
      dat = random_read(0x00, mem_adr);
      PrintToScreen("%04X ", dat);  
      ++mem_adr;
   }
   //delay_ms(500);
   Wait(500);
} // end of while

#endif
}


//********************************************************************************
//* GetCompassHeading - This function reads the electronic compass heading (0-360 degrees)
//********************************************************************************
int Get_Compass_Heading(void)
{
   int Heading = 0; 
   int x, y;

   //PrintToScreen("Start\n\r");    
  
   // Start an I2C transfer 
   i2c_start(); 
 
   // Write the data byte to the I2C bus
   i2c_out_byte(0x42);

   //IdleI2C(); 
   i2c_nack();

   // Write the data byte to the I2C bus
   //MasterWriteI2C(0x41);
   i2c_out_byte(0x41);   //Send command "A"    
   //IdleI2C(); 
   i2c_nack();

   i2c_stop(); 
   Wait(10);    //wait at least 6000 uS 

   i2c_start();        
   //IdleI2C(); 

   // Write the data byte to the I2C bus
   //MasterWriteI2C(0x43);
   i2c_out_byte(0x43);        //Write read command 
   //IdleI2C(); 
   i2c_nack();
   Wait(10);    //wait at least 6000 uS 

   //x = MasterReadI2C();    //Read first byte 
   x = i2c_in_byte();     //Read first byte 
   //IdleI2C(); 
   //AckI2C(); 
   i2c_ack();
   //IdleI2C(); 

  
   //y = MasterReadI2C();    //Read second byte 
   y = i2c_in_byte();     //Read second byte 
   //IdleI2C(); 
   //AckI2C(); 
   //IdleI2C(); 
   //NotAckI2C(); 
   i2c_nack();
   //IdleI2C(); 
        
   i2c_stop(); 

   // Compute heading
   Heading = ((x << 8) + y) / 10;

   return Heading;       
}


void random_write(byte dev_adr, long mem_adr, byte dat)
{
   i2c_start();
   i2c_out_byte(0xa0 | (dev_adr << 1));
   i2c_nack();
   i2c_out_byte((mem_adr >> 8) & 0xff); // high byte of memory address
   i2c_nack();
   i2c_out_byte(mem_adr & 0xff);    // low byte of mem address
   i2c_nack();
   i2c_out_byte(dat);           // and finally the data
   i2c_nack();
   i2c_stop();
   //delay_ms(25); // allow for the programming of the eeprom   
   //pause(25); // allow for the programming of the eeprom
   //Wait(5); // allow for the programming of the eeprom
   Wait(25); // allow for the programming of the eeprom
}

byte random_read(byte dev_adr, long mem_adr)
{
   byte y;
   i2c_start();
   i2c_out_byte(0xa0 | (dev_adr << 1));
   i2c_nack();
   i2c_out_byte((mem_adr >> 8) & 0xff);
   i2c_nack();
   i2c_out_byte(mem_adr & 0xff);
   i2c_nack();
   i2c_start();             // no intermediate stop
   i2c_out_byte(0xa1 | (dev_adr << 1)); // read operation
   i2c_nack();
   y=i2c_in_byte();
   i2c_stop();
   return(y);
}

// Common I2C Routines

byte i2c_in_byte(void)
{
   byte i_byte, n;
   i2c_high_sda();
   for (n=0; n<8; n++)
   {
      i2c_high_scl();

      if (SDA_PIN)
      {
         i_byte = (i_byte << 1) | 0x01; // msbit first
      }
      else
      {
         i_byte = i_byte << 1;
      }
      i2c_low_scl();
   }
   return(i_byte);
}

void i2c_out_byte(byte o_byte)
{
   byte n;
   for(n=0; n<8; n++)
   {
      if(o_byte&0x80)
      {
         i2c_high_sda();
      }
      else
      {
         i2c_low_sda();
      }
      i2c_high_scl();
      i2c_low_scl();
      o_byte = o_byte << 1;
   }
   i2c_high_sda();
}

void i2c_nack(void)
{
   i2c_high_sda();  // data at one
   i2c_high_scl();  // clock pulse
   i2c_low_scl();
}

void i2c_ack(void)
{
   i2c_low_sda();   // bring data low and clock
   i2c_high_scl();
   i2c_low_scl();
   i2c_high_sda();
}


void i2c_start(void)
{
   i2c_low_scl();
   i2c_high_sda();
   i2c_high_scl();  // bring SDA low while SCL is high
   i2c_low_sda();
   i2c_low_scl();
}

void i2c_stop(void)
{
   i2c_low_scl();
   i2c_low_sda();
   i2c_high_scl();
   i2c_high_sda();  // bring SDA high while SCL is high
   // idle is SDA high and SCL high
}

void i2c_high_sda(void)
{
   // bring SDA to high impedance
   SDA_DIR = 1;
   //delay_10us(50);
   delay_10us(5);
}

void i2c_low_sda(void)
{
   SDA_PIN = 0;
   SDA_DIR = 0;  // output a hard logic zero
   //delay_10us(50);
   delay_10us(5);
}

void i2c_high_scl(void)
{
   SCL_DIR = 1;   // high impedance
   //delay_10us(50);
   delay_10us(5);
}

void i2c_low_scl(void)
{
   SCL_PIN = 0;
   SCL_DIR = 0;
   //delay_10us(50);
   delay_10us(5);
}

// LCD routines

void delay_10us(int t)
{
 
   do
   {
        Delay10TCY();
   } while(--t);

/*  

#asm
      BCF STATUS, RP0
DELAY_10US_1:
      CLRWDT
      NOP
      NOP
      NOP
      NOP
      NOP
      NOP
      DECFSZ t, F
      GOTO DELAY_10US_1
#endasm

*/
}

void delay_ms(long t)   // delays t millisecs
{
    Wait(t);
/*
   do
   {
     delay_10us(100);
   } while(--t);
*/

}

int num_to_char(int val)    // converts val to hex character
{
   int ch;
   if (val < 10)
   {
     ch=val+'0';
   }
   else
   {
     val=val-10;
     ch=val + 'A';
   }
   return(ch);
}

void lcd_char(int ch)   // serial output to PIC-n-LCD, 9600 baud
{
   int n, dly;
        // start bit + 8 data bits

/*
  #asm
       BCF STATUS, RP0
       MOVLW 9
       MOVWF n
       BCF STATUS, C

  LCD_CHAR_1:

       BTFSS STATUS, C
       BSF PORTA, TxData
       BTFSC STATUS, C
       BCF PORTA, TxData
       MOVLW 32
       MOVWF dly

  LCD_CHAR_2:
       DECFSZ dly, F
       GOTO LCD_CHAR_2
       RRF ch, F
       DECFSZ n, F
       GOTO LCD_CHAR_1

       BCF PORTA, TxData
       CLRWDT
       MOVLW 96
       MOVWF dly

  LCD_CHAR_3:
       DECFSZ dly, F
       GOTO LCD_CHAR_3
       CLRWDT
   #endasm

*/

}

// LCD routines

void lcd_init(void) // sets TxData in idle state and resets PIC-n-LCD
{
/*
  #asm
        BCF STATUS, RP0
        BCF PORTA, TxData
        BSF STATUS, RP0
        BCF TRISA, TxData
        BCF STATUS, RP0
   #endasm
*/
   lcd_char(0x0c);
   delay_ms(250);
}

void lcd_new_line(void) // outputs 0x0d, 0x0a
{
   lcd_char(0x0d);
   delay_ms(10);    // give the PIC-n-LCD time to perform the
   lcd_char(0x0a);  // new line function
   delay_ms(10);
}


void out_RAM_str(int *s)
{
   while(*s)
   {
      lcd_char(*s);
      ++s;
   }
}

void lcd_hex_byte(int val) // displays val in hex format
{
   int ch;
   ch = num_to_char((val>>4) & 0x0f);
   lcd_char(ch);
   ch = num_to_char(val&0x0f);
   lcd_char(ch);
}

void lcd_dec_byte(int val, int digits)
// displays byte in decimal as either 1, 2 or 3 digits
{
   int d;
   int ch;
   if (digits == 3)
   {
      d=val/100;
      ch=num_to_char(d);
      lcd_char(ch);
   }
   if (digits >1)   // take the two lowest digits
   {
       val=val%100;
       d=val/10;
       ch=num_to_char(d);
       lcd_char(ch);
   }
   if (digits == 1) // take the least significant digit
   {
       val = val%100;
   }

   d=val % 10;
   ch=num_to_char(d);
   lcd_char(ch);
}



















