// Program 24_256_1.C (CCS Info PCM Compiler - PIC16F84)
//
// Illustrates how to write a byte to an address and read a byte from an
// an address.  
//
// May be used for the following EEPROMs;
// 
//  2432 (max adr 0x0fff)
//  2465 (max adr 0x1fff)
//  24128 (max adr 0x3fff)
//  24256 (max adr 0x7fff)
//
// Program writes the 16 values 0xff, 0xfe, etc to locations beginning
// at memory adr 0x0700.  Reads them back and displays on serial LCD.
//
//    PIC16F84                            24LC256
//
// RB1 (term 7) ------------------- SCL (term 6) ----- To Other
// RB2 (term 8) ------------------- SDA (term 5) ----- I2C Devices
//
// Note that the slave address is determined by A2 (term 3), A1
// (term 2) and A0 (term 1) on the 24LC256.  The above SCL and SDA leads
// may be multipled to eight group "1010" devices, each strapped for a
// unique A2 A1 A0 setting.
//
// 10K pullup resistors to +5VDC are required on both signal leads.
//
// Serial LCD is connected to RA0.  Serial data is 9600 baud, inverted.
//
//
// copyright, Peter H. Anderson, Scotland Co, NC, Mar, '99

  
// Include file headers here
/*
#include "flags.h"                                      // Global flags used for MPSIM debugging 
  
#ifdef PIC18C252     
#include <p18C252.h>                    // Register definitions for PIC18C252
#endif

#ifdef PIC18C452     
#include <p18c452.h>                    // Register definitions for PIC18C452
#endif

#ifdef PIC18F452     
#include <p18f452.h>                    // Register definitions for PIC18F452
#endif

#ifdef PIC18F8720     
#include <p18f8720.h>
#endif
 
#ifdef PIC18F6620     
//#include <p18f6620.h>
#endif
*/


#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <i2c.h>                                        // I2C library functions
#include <pwm.h>                                        // PWM library functions
#include <timers.h>                                     // Timer library functions
#include <portb.h>                                      // PORTB library functions                                 
#include <usart.h>                                      // USART library functions
#include <delays.h>                                     // High resolution delay functions
//#include "portutil.h"                                 // Port utility functions
//#include "timeutil.h"
//#include "stdio.h"                                      // Serial I/O support functions
//#include "i2cmsg.h"                                     // I2C networking support functions                                
// #include "my_types.h"                    // General C/C++ types
//#include "lcd.h"                      // LCD Support using SPI2 

//#include <delays.h>
#include "sw_i2c.h"
#include "Main.h"
//#include <portb.h>                                        // PORTB library functions                                 
//#include <delays.h>                                     // High resolution delay functions
//#include "ifi_aliases.h"
//#include "ifi_picdefs.h"
//#include "ifi_default.h"
//#include "ifi_utilities.h"
//#include "user_routines.h"
//#include "lcd.h"
//#include "i2c.h"
 

//********************************************************************************
// Constant Definitions
//********************************************************************************

#define SMALL_I2C_EEPROM                // Use small 24LC01..24LC16B Serial EEPROM(s)
//#define LARGE_I2C_EEPROM              // Use large 24LC32..24LC256 Serial EEPROM(s)

//#define TEST                                          // Flag used for debug I/O
//#define USART_TEST                                    // Test serial I/O with this application

#define MASTER_TEST_A                                 // Select the I2C Master Controller test A
//#define MASTER_TEST_0                                 // Select the I2C Master Controller test #0
//#define MASTER_TEST_1                                  // Select the I2C Master Controller test #1
//#define MASTER_TEST_2                                 // Select the I2C Master Controller test #2
//#define MASTER_TEST_3                                 // Select the I2C Master Controller test #3
//#define MASTER_TEST_4                                 // Select the I2C Master Controller test #4


//#define EEPROM_TEST_0                                 // Select EEPROM test #0 for 24LC01A devices
//#define EEPROM_TEST_1                                 // Select EEPROM test #1 for 24LC01A devices
//#define EEPROM_TEST_2                                 // Select EEPROM test #2 for 24LC32A devices  
//#define EEPROM_TEST_3                                 // Select EEPROM test #3 for 24LC16A devices (This version works !)
//#define EEPROM_TEST_4                                 // Select EEPROM test #4 for 24LC256A devices
//#define EEPROM_TEST_5                                 // Select EEPROM test #5 for 24LC256A devices
 
//#define SMART_PERIPHERAL_TEST                         // Select the smart peripheral test 

#define RAMP_SIZE  32                                   // Size of data buffer
#define SYSTEM_FREQUENCY 20                             // System frequency, Fs = 20 MHz

//#define BAUD      100                                 // Desired Baud Rate in Kbps (100 KHz)
#define BAUD        400                                 // Desired Baud Rate in Kbps (400 KHz)

//#define FOSC      4000                                //  Oscillator Clock in KHz (4 MHz) 
#define FOSC        20000                               // Oscillator Clock in KHz (20 MHz) 

// Compute timer clock frequency which depends on the system frequency.  This constant is 
// used by DelayUs function. 

#define CLOCK_FREQUENCY   (SYSTEM_FREQUENCY / 4)        //  Fc = Fs / 4 
 
#define TIME_SCALE_FACTOR 100                                         // 100 micro-second granularity for delays

 
#define TRUE  1                                         // Boolean TRUE value
#define FALSE 0                                         // Boolean FALSE value

#define READ  1                                         // I2C read operation
#define WRITE 0                                         // I2C write operation


// Maximum address that may be used for the following EEPROM(s) :

// 

//      2432 (max adr 0x0fff)

//      2465 (max adr 0x1fff)

//      24128 (max adr 0x3fff)

//      24256 (max adr 0x7fff)

//#define MAXIMUM_ADDRESS 0x0FFF                    // Maximum address for a 24LC32 Serial EEPROM
//#define MAXIMUM_ADDRESS 0x7FFF                    // Maximum address for a 24LC256 Serial EEPROM
#define MAXIMUM_ADDRESS 32                          // Maximum address for a 24LCXXX Serial EEPROM

#define WRITE_ADDRESS    0xA0                       // I2C Serial EEPROM write device address  
#define READ_ADDRESS     0xA1                       // I2C Serial EEPROM read device address  

//#define WRITE_ADDRESS    0xD6                     // I2C write device address (unused by other I2C devices)
//#define READ_ADDRESS     0xD7                     // I2C read device address (unused by other I2C devices)

//*******************************************************************************
// Define port bit definitions here
//*******************************************************************************

#define INPUT    1                                     // Input pin direction
#define OUTPUT   0                                     // Output pin direction
 
 
void main(void);

#define TxData 0    // RA.0 for serial LCD

//********************************************************************************
//* main - Receives I2C commands from the Host PC and sends them to the I2C slave
//* device.
//********************************************************************************
   
#ifdef TEST 
void main(void)
{
   int i,j;                     //  Loop index

  
   SetupTimers();                                         // Setup timers and software
 
   // Configure USART for 20 MHz, 9600 Baud, 8-Bits, No parity, Asynchronous
   
   InitializeUsart();

    printf("PIC18C452 I2C Master Controller Application... \r\n");

   // Display message to operator

   PrintToScreen("PIC18C452 I2C Master Controller Application... \r\n");

   PrintToScreen("Delay 5 seconds ...\r\n");
   Wait(5000);
   PrintToScreen("End of 5 seconds.\r\n");

 

   // Initialize I2C variables and hardware



    PrintToScreen("Calling init_i2c ...\r\n");
   init_i2c();


}
#endif

 
void init_i2c(void)
{
   long mem_adr;
   byte dat, n;
   int i;
   //int compassAddress = 0x42 >> 1;  // From datasheet compass address is 0x42
   int compassAddress = 0x42;  // From datasheet compass address is 0x42
                                 // shift the address 1 bit right, the Wire library only needs the 7
                                 // most significant bits for the address
   int reading = 0; 
   int Direction = 0; 
   int x, y;

//***********************************************************************************************
// Initialize the pins used for the I2C interface
// configure port A for all digital inputs

//OpenADC(ADC_FOSC_32 &
// ADC_RIGHT_JUST & ADC_0ANA_0REF, ADC_CH0 & ADC_CH1 & ADC_CH2 & ADC_CH3 & ADC_CH4 & ADC_CH5 & ADC_CH6 & ADC_CH7 & ADC_INT_OFF);
//OpenADC(ADC_0ANA_0REF, ADC_INT_OFF);
//OpenADC(ADC_0ANA, ADC_INT_OFF);

// Configure PORTB interrupt with Interrupt on change off and pull-up resistors disabled

//OpenPORTB(PORTB_CHANGE_INT_OFF & PORTB_PULLUPS_OFF);

// Configure PORT E for 8 Bits Digital I/O 

//PSPCONbits.PSPMODE = 0;

//The pullups on the interrupt pins are internal to the PIC micro. If that its the case, you can disable them by setting INTCON2.NOT_RBPU = 1.
//Note: that this also disables the pullups for RB0 & RB1, which might effect the internal operation of the microcontroller. I'm not sure if 
// those lines are used for inter-processor communication. this is gleaned from the PIC18F8520 data sheet, and from examining ifi_aliases.h & ifi_picdefs.h

//INTCON2.NOT_RBPU = 1;


// Disable serial port  
RCSTA2bits.SPEN = 0;    // Serial Port disable [249]
                            //  0: serial port is disabled
                            //  1: serial port is enabled

//***********************************************************************************************
/*
    PrintToScreen("trace 1 - Test SCL line \r\n");
   // Test the SCL line
   while (1)
   {
        i2c_low_scl();
        //Wait(1);
        i2c_high_scl();
        //Wait(1);
   }


    PrintToScreen("trace 2 - Test SDA line \r\n");
  // Test the SDA line
   while (1)
   {
        i2c_low_sda();
        //Wait(1);
        i2c_high_sda();
        //Wait(1);
   }
*/

 
// Test the Honeywell HMC6352 I2C Electronic Compass
PrintToScreen("Test the Honeywell HMC6352 I2C Electronic Compass...\r\n");
Wait(2000);
PrintToScreen("trace 1\r\n");
for (i=0; i<100; i++)
{
    PrintToScreen("This is a test %d times\r\n",i);
} 
 
    //****************************************************************************************************************
    //****************************************************************************************************************
    //* Compass test - Read the I2C compass
    //****************************************************************************************************************
    //****************************************************************************************************************


    //main loop code goes here 
    PrintToScreen("Read the I2C Compass and report current heading in degrees  ... \r\n");

    while(1)    
    { 
        //ClrWdt(); 
    
        PrintToScreen("Start\n\r");    
        i2c_start(); 
        //IdleI2C(); 

        // Write the data byte to the I2C bus
        //MasterWriteI2C(0x42);
        i2c_out_byte(0x42);
        //IdleI2C(); 
        i2c_nack();

        // Write the data byte to the I2C bus
        //MasterWriteI2C(0x41);
        i2c_out_byte(0x41);   //Send command "A"    
       //IdleI2C(); 
        i2c_nack();

        i2c_stop(); 
        Wait(10);    //wait at least 6000 uS 

        i2c_start();        
       //IdleI2C(); 

        // Write the data byte to the I2C bus
        //MasterWriteI2C(0x43);
        i2c_out_byte(0x43);        //Write read command 
        //IdleI2C(); 
        i2c_nack();

        Wait(10);    //wait at least 6000 uS 

        //x = MasterReadI2C();    //Read first byte 
        x = i2c_in_byte();     //Read first byte 
        //IdleI2C(); 
        //AckI2C(); 
        i2c_ack();
        //IdleI2C(); 

  
        //y = MasterReadI2C();    //Read second byte 
        y = i2c_in_byte();     //Read second byte 
        //IdleI2C(); 
        //AckI2C(); 
        //IdleI2C(); 
        //NotAckI2C(); 
        i2c_nack();
        //IdleI2C(); 
        
        i2c_stop(); 
        Direction = ((x << 8) + y) / 10;

        PrintToScreen("Direction (deg.) = %i\n\r", Direction); 

        Wait(200);        
    } 

    

//***********************************************************************************

// Write data to 24LC512 Serial I2C EEPROM


while(1)
{
    for (i=0; i<256; i++)
    //for (i=0; i<20; i++)
    {
        PrintToScreen("write: i = %d, dat = %d\r\n", i, i);
        random_write(0x00, i, 256 - (byte) i);
 
    }

    for (i=0; i<256; i++)
    //for (i=0; i<20; i++)
    {
        dat = random_read(0x00, i);
        PrintToScreen("read: i = %d, dat = %d\r\n", i, dat);

    }
    //delay_ms(500);
}

while(1)
{

   mem_adr=0x0700;
   for(n=0; n<16; n++)
   {
      dat = 0xff-n;
      PrintToScreen("trace 2\r\n");
      random_write(0x00, mem_adr, dat);
      ++mem_adr;
   }
    // now, read the data back and display
   lcd_init();
   mem_adr=0x0700;

   PrintToScreen("trace 3\r\n");

   for(n=0; n<16; n++)
   {
      if ((n!=0) && ((n%4) == 0))
      {
         PrintToScreen("/r/n");
      }
      PrintToScreen("trace 4\r\n");
      dat = random_read(0x00, mem_adr);
      PrintToScreen("%04X ", dat);  
      ++mem_adr;
   }
   //delay_ms(500);
   Wait(500);
} // end of while
}

void random_write(byte dev_adr, long mem_adr, byte dat)
{
   i2c_start();
   i2c_out_byte(0xa0 | (dev_adr << 1));
   i2c_nack();
   i2c_out_byte((mem_adr >> 8) & 0xff); // high byte of memory address
   i2c_nack();
   i2c_out_byte(mem_adr & 0xff);    // low byte of mem address
   i2c_nack();
   i2c_out_byte(dat);           // and finally the data
   i2c_nack();
   i2c_stop();
   //delay_ms(25); // allow for the programming of the eeprom   
   //pause(25); // allow for the programming of the eeprom
   //Wait(5); // allow for the programming of the eeprom
   Wait(25); // allow for the programming of the eeprom
}

byte random_read(byte dev_adr, long mem_adr)
{
   byte y;
   i2c_start();
   i2c_out_byte(0xa0 | (dev_adr << 1));
   i2c_nack();
   i2c_out_byte((mem_adr >> 8) & 0xff);
   i2c_nack();
   i2c_out_byte(mem_adr & 0xff);
   i2c_nack();
   i2c_start();             // no intermediate stop
   i2c_out_byte(0xa1 | (dev_adr << 1)); // read operation
   i2c_nack();
   y=i2c_in_byte();
   i2c_stop();
   return(y);
}

// Common I2C Routines

byte i2c_in_byte(void)
{
   byte i_byte, n;
   i2c_high_sda();
   for (n=0; n<8; n++)
   {
      i2c_high_scl();

      if (SDA_PIN)
      {
         i_byte = (i_byte << 1) | 0x01; // msbit first
      }
      else
      {
         i_byte = i_byte << 1;
      }
      i2c_low_scl();
   }
   return(i_byte);
}

void i2c_out_byte(byte o_byte)
{
   byte n;
   for(n=0; n<8; n++)
   {
      if(o_byte&0x80)
      {
         i2c_high_sda();
      }
      else
      {
         i2c_low_sda();
      }
      i2c_high_scl();
      i2c_low_scl();
      o_byte = o_byte << 1;
   }
   i2c_high_sda();
}

void i2c_nack(void)
{
   i2c_high_sda();  // data at one
   i2c_high_scl();  // clock pulse
   i2c_low_scl();
}

void i2c_ack(void)
{
   i2c_low_sda();   // bring data low and clock
   i2c_high_scl();
   i2c_low_scl();
   i2c_high_sda();
}


void i2c_start(void)
{
   i2c_low_scl();
   i2c_high_sda();
   i2c_high_scl();  // bring SDA low while SCL is high
   i2c_low_sda();
   i2c_low_scl();
}

void i2c_stop(void)
{
   i2c_low_scl();
   i2c_low_sda();
   i2c_high_scl();
   i2c_high_sda();  // bring SDA high while SCL is high
   // idle is SDA high and SCL high
}

void i2c_high_sda(void)
{
   // bring SDA to high impedance
   SDA_DIR = 1;
   //delay_10us(50);
   delay_10us(5);
}

void i2c_low_sda(void)
{
   SDA_PIN = 0;
   SDA_DIR = 0;  // output a hard logic zero
   //delay_10us(50);
   delay_10us(5);
}

void i2c_high_scl(void)
{
   SCL_DIR = 1;   // high impedance
   //delay_10us(50);
   delay_10us(5);
}

void i2c_low_scl(void)
{
   SCL_PIN = 0;
   SCL_DIR = 0;
   //delay_10us(50);
   delay_10us(5);
}

// LCD routines

void delay_10us(int t)
{
 
   do
   {
        Delay10TCY();
   } while(--t);

/*  

#asm
      BCF STATUS, RP0
DELAY_10US_1:
      CLRWDT
      NOP
      NOP
      NOP
      NOP
      NOP
      NOP
      DECFSZ t, F
      GOTO DELAY_10US_1
#endasm

*/
}

void delay_ms(long t)   // delays t millisecs
{
    Wait(t);
/*
   do
   {
     delay_10us(100);
   } while(--t);
*/

}

int num_to_char(int val)    // converts val to hex character
{
   int ch;
   if (val < 10)
   {
     ch=val+'0';
   }
   else
   {
     val=val-10;
     ch=val + 'A';
   }
   return(ch);
}

void lcd_char(int ch)   // serial output to PIC-n-LCD, 9600 baud
{
   int n, dly;
        // start bit + 8 data bits

/*
  #asm
       BCF STATUS, RP0
       MOVLW 9
       MOVWF n
       BCF STATUS, C

  LCD_CHAR_1:

       BTFSS STATUS, C
       BSF PORTA, TxData
       BTFSC STATUS, C
       BCF PORTA, TxData
       MOVLW 32
       MOVWF dly

  LCD_CHAR_2:
       DECFSZ dly, F
       GOTO LCD_CHAR_2
       RRF ch, F
       DECFSZ n, F
       GOTO LCD_CHAR_1

       BCF PORTA, TxData
       CLRWDT
       MOVLW 96
       MOVWF dly

  LCD_CHAR_3:
       DECFSZ dly, F
       GOTO LCD_CHAR_3
       CLRWDT
   #endasm

*/

}

// LCD routines

void lcd_init(void) // sets TxData in idle state and resets PIC-n-LCD
{
/*
  #asm
        BCF STATUS, RP0
        BCF PORTA, TxData
        BSF STATUS, RP0
        BCF TRISA, TxData
        BCF STATUS, RP0
   #endasm
*/
   lcd_char(0x0c);
   delay_ms(250);
}

void lcd_new_line(void) // outputs 0x0d, 0x0a
{
   lcd_char(0x0d);
   delay_ms(10);    // give the PIC-n-LCD time to perform the
   lcd_char(0x0a);  // new line function
   delay_ms(10);
}


void out_RAM_str(int *s)
{
   while(*s)
   {
      lcd_char(*s);
      ++s;
   }
}

void lcd_hex_byte(int val) // displays val in hex format
{
   int ch;
   ch = num_to_char((val>>4) & 0x0f);
   lcd_char(ch);
   ch = num_to_char(val&0x0f);
   lcd_char(ch);
}

void lcd_dec_byte(int val, int digits)
// displays byte in decimal as either 1, 2 or 3 digits
{
   int d;
   int ch;
   if (digits == 3)
   {
      d=val/100;
      ch=num_to_char(d);
      lcd_char(ch);
   }
   if (digits >1)   // take the two lowest digits
   {
       val=val%100;
       d=val/10;
       ch=num_to_char(d);
       lcd_char(ch);
   }
   if (digits == 1) // take the least significant digit
   {
       val = val%100;
   }

   d=val % 10;
   ch=num_to_char(d);
   lcd_char(ch);
}











